import * as i0 from '@angular/core';
import { Injectable, EventEmitter, Component, ChangeDetectionStrategy, Input, Output, APP_INITIALIZER, NgModule } from '@angular/core';
import { BehaviorSubject, filter, switchMap, from, of, take, tap, map, catchError, throwError } from 'rxjs';
import * as i2 from '@angular/common/http';
import { HttpClientModule } from '@angular/common/http';
import * as i3 from '@angular/common';
import { CommonModule } from '@angular/common';
import { openDB } from 'idb/with-async-ittr';

const NGX_HTTP_GET_SHELL_DB = 'ngx-http-get-shell';
const NGX_HTTP_GET_SHELL_STORE = 'ngx-http-get-shell-cache';
const NGX_HTTP_GET_SHELL_INDEXED_KEY = 'expires';
/**
 * This service handles client-side indexed db storage read/write APIs
 */
class IndexedDbService {
    constructor() {
        this._db = IndexedDbService.db;
    }
    /**
     * Cleans up the database by removing expired items
     */
    static async cleanup(db) {
        const deletes = [];
        const tx = db.transaction(NGX_HTTP_GET_SHELL_STORE, 'readwrite');
        const now = Date.now();
        const index = tx.store.index(NGX_HTTP_GET_SHELL_INDEXED_KEY);
        for await (const record of index.iterate(null, 'next')) {
            if (record.value.expires < now) {
                deletes.push(record.delete());
            }
            else {
                break;
            }
        }
        await Promise.all(deletes);
        await tx.done;
    }
    /**
     * Adds an item to the database
     * @param key The key to store the value under
     * @param value The value to store
     * @param expires The time in milliseconds until the value expires
     * @returns The stored key
     */
    setItem(key, value, expires) {
        const NgxHttpGetStorable = {
            value,
            expires,
        };
        return this._db.put(NGX_HTTP_GET_SHELL_STORE, NgxHttpGetStorable, key);
    }
    /**
     * Gets an item from the database
     * @param key The key to retrieve the value from
     * @returns The stored value
     */
    async getItem(key) {
        const NgxHttpGetStorable = await this._db.get(NGX_HTTP_GET_SHELL_STORE, key);
        return NgxHttpGetStorable?.value;
    }
    /**
     * Deletes an item from the database
     * @param key The key to delete
     */
    removeItem(key) {
        return this._db.delete(NGX_HTTP_GET_SHELL_STORE, key);
    }
    /**
     * Clear all items from the database
     */
    clearAll() {
        return this._db.clear(NGX_HTTP_GET_SHELL_STORE);
    }
    /**
     * The the indexed db instance used by this service
     * @returns Indexed db instance
     */
    getDb() {
        return this._db;
    }
}
IndexedDbService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: IndexedDbService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
IndexedDbService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: IndexedDbService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: IndexedDbService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return []; } });

class NgxHttpGetShellComponent {
    constructor(_storage, _http) {
        this._storage = _storage;
        this._http = _http;
        /**
         * Max Stale duration in milliseconds, default is 0, which means no cache.
         */
        this.maxStale = 0;
        /**
         * Emitted when the data is loaded and processed.
         */
        this.dataLoaded = new EventEmitter();
        this.data$ = new BehaviorSubject(null);
        this._url$ = new BehaviorSubject(null);
    }
    ngOnChanges(changes) {
        if (changes['url']?.currentValue) {
            this.data$.next(null);
            this._url$.next(changes['url'].currentValue);
        }
    }
    ngOnInit() {
        this._subscribeToUrlChange();
    }
    _subscribeToUrlChange() {
        this._sub = this._url$
            .pipe(filter((url) => !!url), switchMap((url) => from(this._storage.getItem(url)).pipe(switchMap((data) => data ? of(data) : this._http.get(url).pipe(take(1), tap((data) => {
            const maxStale = Number(this.maxStale);
            if (maxStale > 0)
                this._storage.setItem(url, data, Date.now() + maxStale);
        }))))), map((response) => this.dataProcessor ? this.dataProcessor(response) : response), tap((data) => {
            this.data$.next(data);
            this.dataLoaded.emit(data);
        }), catchError((err) => {
            this.errorHandler?.call(this, err);
            return throwError(() => err);
        }))
            .subscribe();
    }
    ngOnDestroy() {
        this._sub.unsubscribe();
    }
}
NgxHttpGetShellComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: NgxHttpGetShellComponent, deps: [{ token: IndexedDbService }, { token: i2.HttpClient }], target: i0.ɵɵFactoryTarget.Component });
NgxHttpGetShellComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "14.0.3", type: NgxHttpGetShellComponent, selector: "ngx-http-get-shell", inputs: { url: "url", maxStale: "maxStale", dataProcessor: "dataProcessor", errorHandler: "errorHandler", templateRef: "templateRef" }, outputs: { dataLoaded: "dataLoaded" }, usesOnChanges: true, ngImport: i0, template: "<ng-container *ngIf=\"(data$ | async) as data\">\n  <ng-template [ngTemplateOutlet]=\"templateRef\" [ngTemplateOutletContext]=\"{ $implicit: data }\"></ng-template>\n</ng-container>\n", dependencies: [{ kind: "directive", type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i3.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "pipe", type: i3.AsyncPipe, name: "async" }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: NgxHttpGetShellComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ngx-http-get-shell', changeDetection: ChangeDetectionStrategy.OnPush, template: "<ng-container *ngIf=\"(data$ | async) as data\">\n  <ng-template [ngTemplateOutlet]=\"templateRef\" [ngTemplateOutletContext]=\"{ $implicit: data }\"></ng-template>\n</ng-container>\n" }]
        }], ctorParameters: function () { return [{ type: IndexedDbService }, { type: i2.HttpClient }]; }, propDecorators: { url: [{
                type: Input
            }], maxStale: [{
                type: Input
            }], dataProcessor: [{
                type: Input
            }], errorHandler: [{
                type: Input
            }], templateRef: [{
                type: Input
            }], dataLoaded: [{
                type: Output
            }] } });

class NgxHttpGetShellModule {
    static forRoot() {
        return {
            ngModule: NgxHttpGetShellModule,
            providers: [
                {
                    provide: APP_INITIALIZER,
                    useFactory: () => async function () {
                        const db = await openDB(NGX_HTTP_GET_SHELL_DB, 1, {
                            upgrade(db) {
                                const httpGetCacheStore = db.createObjectStore(NGX_HTTP_GET_SHELL_STORE);
                                httpGetCacheStore.createIndex(NGX_HTTP_GET_SHELL_INDEXED_KEY, NGX_HTTP_GET_SHELL_INDEXED_KEY);
                            },
                        });
                        IndexedDbService.cleanup(db);
                        IndexedDbService.db = db;
                    },
                    multi: true,
                },
                IndexedDbService,
            ],
        };
    }
}
NgxHttpGetShellModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: NgxHttpGetShellModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
NgxHttpGetShellModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.0.3", ngImport: i0, type: NgxHttpGetShellModule, declarations: [NgxHttpGetShellComponent], imports: [CommonModule, HttpClientModule], exports: [NgxHttpGetShellComponent] });
NgxHttpGetShellModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: NgxHttpGetShellModule, imports: [CommonModule, HttpClientModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: NgxHttpGetShellModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule, HttpClientModule],
                    declarations: [NgxHttpGetShellComponent],
                    exports: [NgxHttpGetShellComponent],
                }]
        }] });

/*
 * Public API Surface of ngx-http-get-shell
 */

/**
 * Generated bundle index. Do not edit.
 */

export { IndexedDbService, NGX_HTTP_GET_SHELL_DB, NGX_HTTP_GET_SHELL_INDEXED_KEY, NGX_HTTP_GET_SHELL_STORE, NgxHttpGetShellComponent, NgxHttpGetShellModule };
//# sourceMappingURL=ngx-http-get-shell.mjs.map
